<?php

/*
 *  ShoutPSP v1.0
 *  (c) 2006, Danil Haveman, psp@blaat.cjb.net 
 *
 *  ShoutPSP is a piece of software the will connect to a shoutcast server
 *  and fake it's a file download instead of a stream. This means your PSP  
 *  will think it's a normal file-download (of 1GB large). If you put such
 *  a file in an rss feed it allows you to listen to your favorite shoutcast
 *  servers on your PSP anywhere and even with firmware 2.7  
 *
 *  Parameters:
 *  ip - the ip-address of the Shoutcast server
 *  port - the portnumber for the Shoutcast stream  
 *  stream - the file-location of the stream
 *  
 *  For example a stream at:
 *    
 *    http://64.236.34.67:80/stream/1003
 *    
 *  Would be added to the RSS as:
 *  
 *    http://someserver/ShoutPSP.php?ip=64.236.34.67&port=80&stream=/stream/1003
 *
 *  There is a bug in the PSP's RSS parser in that it cannot at this moment
 *  handle xml-entities. So the & sign should be entered in the xml as an & and
 *  not as an &amp; as is specified by the xml specification.
 *  
 *  Anyone can use this software on their servers. And you are in no way
 *  restricted in it's use. However I would like to be notified if you use it
 *  in some special way, or if you make some cool modifications to it. Read
 *  below for the usage license, copyright and disclaimer.  
 */

/*
 *  Copyright (c) 2006 Danil Haveman
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a copy 
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 *  copies of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included in 
 *  all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 *  IN THE SOFTWARE.
 */ 


// Define the fake download size
$size=1000000000;

// The PSP first checks to see what the server supports using a HEAD request. 
// It requires the Accept-Ranges and Content-Length headers to work.
if ($_SERVER["REQUEST_METHOD"]=="HEAD") {
	header("Last-Modified: Tue, 16 May 2006 00:00:00 GMT");
  header("Cache-Control: no-cache, must-revalidate");
  header("Expires: Mon, 31 Jul 1979 00:00:00 GMT");
	header("ETag: \"580119-be000-720ae380\"");
	header("Accept-Ranges: bytes");
	header("Content-Length: ".$size, true);
	header("Content-Type: audio/mpeg");
	
	// Apache filters all Content-Length headers when a HEAD request is issued
	// to a script. So we flush the headers to apache which will strip the header
	// and afterwards send one final header with the Content-Length. Apache will
	// think it has already stripped it so will ignore it. 
	flush();
	header("Content-Length:".$size);
	
	// When a HEAD request is issued we should only give back headers.
	exit;
}

// The PSP then begins a normal file download or does a range request
if(!isset($_SERVER['HTTP_RANGE'])) {

  // A normal file download
	header("Last-Modified: Tue, 16 May 2006 00:00:00 GMT");
  header("Cache-Control: no-cache, must-revalidate");
  header("Expires: Mon, 31 Jul 1979 00:00:00 GMT");
	header("ETag: \"580119-be000-720ae380\"");
	header("Accept-Ranges: bytes");
	header("Content-Length: ".$size, true);
	header("Content-Type: audio/mpeg");
} else {

  // A range is given
  // The PSP allways gives a range in the format "bytes=xx-"
  // so we strip the last character from the response and split the string
  // at the "=" which leaves us with just the "xx" part.
	$range=explode("=",$_SERVER['HTTP_RANGE']);
	$range = substr($range[1],0,-1);
	
  header("HTTP/1.1 206 Partial Content");
	header("Last-Modified: Tue, 16 May 2006 00:00:00 GMT");
  header("Cache-Control: no-cache, must-revalidate");
  header("Expires: Mon, 31 Jul 1979 00:00:00 GMT");
	header("ETag: \"580119-be000-720ae380\"");
	header("Accept-Ranges: bytes");
	header("Content-Range: bytes ".$range."-".($size)."/".$size);
	header("Content-Type: audio/mpeg");
	
	// We should give back a Content-length but Apache won't let us. I tried the
	// flush trick but this gives some pretty weird results because apache then
	// decides to do something with the range aswell and feeds up some pretty
	// weird numbers to the PSP. The PSP also ignores the Content-Length at this
	// point so it isn't essential anyway.	
}


// This function gets the mp3 stream from the specified shoutcast server
// and dumps it to the PSP which at this point already thinks it's getting
// a normal file.
function getcontent($server, $port, $file) {
  $downloadBuffer = "";
  $fileHandle = fsockopen($server, $port) or die("No connection");
  if (!$fileHandle) {
    return "Unknown error";
  } else {
  
    // We identify ourselves as winampMPEG/2.7; some servers will reject us if
    // we don't. Also we do a HTTP/1.0 request with an Icy-MetaData:0 so the
    // server will think we're an old client. This will prevent streaming ID3
    // data from entering into the stream. Some servers will not allow this
    // so I am currently working on a fix.
    $streamRequest = "GET $file HTTP/1.0\r\nIcy-MetaData:0\r\nAccept: */*\r\nUser-Agent: winampMPEG/2.7\r\nHost: $ip:$port\r\nConnection: Keep-Alive\r\n\r\n";
    fputs($fileHandle, $streamRequest) or die("Could not request data");

    // The Shoutcast server usually expects us to read 8192 byte blocks
    $metaint = 8192;

    // Parse the header
    $header = true;
    while (!feof($fileHandle) && $header) {
      $downloadBuffer = fgets($fileHandle);
      if (strpos($downloadBuffer, "meta")!==false) {
        $downloadBuffer = explode(": ", $cont);
        if ((int)$downloadBuffer[1]>0) $metaint = (int)$downloadBuffer[1];
      }
      if (($downloadBuffer=="\r\n") || ($downloadBuffer=="\n")) {
        $header = false;
      }
    }
    while (!feof($fileHandle)) {
      $downloadBuffer = fread($fileHandle, $metaint);
      echo substr($downloadBuffer,0,$metaint);
    }
    fclose($fileHandle);
    return "All is well";
  }
}

// Start the actual stream
getcontent($_REQUEST["ip"], $_REQUEST["port"], $_REQUEST["stream"]);
?>
